@Filename: UrtCfg.s
@Part: ADuC702x.
@Created: 20/04/2004
@Updated: 28/06/2005
@Author: Eckart Hartmann.
@This file is in beta sampling.

.include        "ADuC7020.equ"
.ARM

@ UrtCfg==========Configures UART.
@ C Function prototype: int UrtCfg(int iChan, int iBaud, int iFormat);
@iChan:{0,1}
@iBaud:{URT_6,URT_12,URT_24,URT_48,URT_96,URT_192,URT_384,URT_576,URT_1152,URT_2304,URT_4608}
@iFormat:{URT_68|URT_78|URT_STP2|URT_PE|URT_PODD|URT_PS|URT_BR}
@ Description of Function: Reads the status byte of the UART.
@ User interface:
@   Set iChan to 0 to use pins SPMUX0 and SPMUX1 or
@    set iChan to 1 to use pins SPMUX8 and SPMUX9
@   Set iBaud to the baudrate required:
@       Either set iBaud to one of 0 to 10 for one of the 
@        baudrates listed above (URT_nx100). In this case
@        the clock must be 45088768Hz+-1%.
@       Alternatively set iBaud to CLK/Baudrate 
@        (e.g. 45088768/9600 = 4697).
@   Set iFormat to the data format required (COMCON0):
@       COMCON0.0 = URT_68\_/ Data length 5, 6, 7 or 8 bits.
@       COMCON0.1 = URT_78/ \
@       COMCON0.2 = URT_STP2 -> 2 stop bits.
@       COMCON0.3 = URT_PE -> Parity used.
@       COMCON0.4 = URT_PODD -> Odd parity.
@       COMCON0.5 = URT_PS -> Sticky parity.
@       COMCON0.6 = URT_BR -> Break (Set SOUT to 0).
@       COMCON0.7 to COMCON.31 -> Ignored. 
@   Call UrtCfg.
@   Returns value of COMSTA0: See UrtSta function for bit details.
@ Robustness:
@   User must check that rounding errors plus clock errors 
@    are acceptable.
@   Only the SIN and SOUT pins of the chosen channel are effected.
@   The baudrate is set using the current CD bits. The baudrate 
@   will change if the CD bits are changed subsequently.
@ Side effects: 
@   Corrupts r1, r2 and r3.
@   Reconfigures the UART.

.GLOBAL UrtCfg
UrtCfg:
        mov     r3,#0xff000000          @r3 = F_BASE;
        mov     r3,r3,ASR#12
        cmp     r0,#0                   @if(iChan==0)
        bne     Urt1                    @ {
        ldr     r0,[r3,#GP1CON_OFFSET]  @ GP1CON &= 0xffffff00;
        and     r0,r0,#0xffffff00
        orr     r0,r0,#0x00000011       @ GP1CON |= 0x00000011;
        str     r0,[r3,#GP1CON_OFFSET]  @ }
        b       UrtBr                   @else
Urt1:   ldr     r0,[r3,#GP1CON_OFFSET]  @ {
        and     r0,r0,#0x0fffffff       @ GP0CON &= 0x0fffffff;
        orr     r0,r0,#0x10000000       @ GP0CON |= 0x10000000;
        str     r0,[r3,#GP1CON_OFFSET]
        ldr     r0,[r3,#GP1CON_OFFSET]
        and     r0,r0,#0xfffffff0       @ GP0CON &= 0xfffffff0;
        orr     r0,r0,#0x00000001       @ GP0CON |= 0x00000001;
        str     r0,[r3,#GP1CON_OFFSET]  @ }
UrtBr:  cmp     r1,#10                  @if(iBaud>10)
        bls     UrtLst
        mov     r1,r1,LSR#5             @ r1 = iBaud>>5;
        b       UrtDl
UrtLst: cmp     r1,#6                   @else { if(iBaud<6)
        ldrls   r0,=0x92c               @ r0 = 2348@ //45088768/(600*16*2)
        ldrhi   r0,=0xc3b               @ else r0 = 3131; } //45088768*128/(57600*16*2)
        mov     r1,r0,LSR r1            @r1 = r0>>iBaud;
UrtDl:  mov     r3,r3,LSL#4             @r3 = Z_BASE;
        mov     r0,#0                   @COMDIV2 = 0;
        str     r0,[r3,#COMDIV2_OFFSET]
        str     r0,[r3,#COMIEN0_OFFSET] @COMIEN0 = 0;
        ldr     r0,[r3,#POWCON_OFFSET]  @r1 >>= CD;
        and     r0,r0,#7
        mov     r1,r1,LSR r0
        mov     r0,#0x83                @COMDIV0-1 = r1.
        strb    r0,[r3,#COMCON0_OFFSET]
        strb    r1,[r3,#COMDIV0_OFFSET]
        mov     r1,r1,LSR#8
        strb    r1,[r3,#COMDIV1_OFFSET]
        and     r1,r2,#0x7F             @COMCON0 = iFormat&0x7f;
        strb    r1,[r3,#COMCON0_OFFSET]
        bx      lr
        
